import json
import uuid
from unittest.mock import patch

from _pytest.monkeypatch import MonkeyPatch

import rasax.community.utils as rasa_x_utils
from rasa.core.trackers import EventVerbosity
from rasax.community.services.analytics_service import AnalyticsService
from rasax.community.services.event_consumers.event_consumer import EventConsumer
from rasax.community.services.event_service import EventService
from rasax.community.services.logs_service import LogsService
from rasax.community.services.model_service import ModelService
from tests.unit.conftest import user_event, insert_test_model


def test_e2e_message(
    event_service: EventService,
    logs_service: LogsService,
    analytics_service: AnalyticsService,
    model_service: ModelService,
    monkeypatch: MonkeyPatch,
):
    # test that a message is processed correctly by the event service

    # create user event with unknown text so it ends up in NLU logs
    conversation_id = "my-id-444"
    test_project = "default"
    text = f"hello from {uuid.uuid4().hex}"
    _user_event = user_event(
        conversation_id, text=text, timestamp=1, project=test_project
    )

    # for in-memory sqlite tests, we need to make sure the services connected to the
    # event consumer use the same database session, hence we mock the initialise method
    def mocked_initialise_services(self):
        self.event_service = event_service
        self.logs_service = logs_service
        self.analytics_service = analytics_service

    with patch.object(EventConsumer, "initialise_services", mocked_initialise_services):
        event_consumer = EventConsumer()

    # mock utils method to skip reporting for enterprise version
    monkeypatch.setattr(rasa_x_utils, "is_enterprise_installed", lambda: False)

    # we need to insert a model so the log can be associated with it
    test_model = "some-model001"
    insert_test_model(test_project, test_model, model_service)

    # receive the event in the event consumer
    event_consumer.log_event(json.dumps(_user_event))

    # ensure the event is in the tracker received from db
    tracker = event_service.get_tracker_for_conversation(conversation_id)
    tracker_state = tracker.current_state(EventVerbosity.ALL)
    assert tracker_state["events"][0]["text"] == text

    # also make sure the NLU log has been created
    logs = logs_service.get_suggestions().result
    assert any(
        log.get("model") == test_model and log.get("user_input", {}).get("text") == text
        for log in logs
    )

    # make sure analytics service has processed the event
    # exclude all conversation IDs apart from the one used in this test
    all_conversation_ids = set(event_service.sender_ids())
    conversation_ids_to_exclude = all_conversation_ids - {conversation_id}
    analytics_result = analytics_service.calculate_analytics(
        start=0, until=10, sender_ids_to_exclude=list(conversation_ids_to_exclude)
    )
    # there should be at least one user message
    assert sum(analytics_result["user_messages"])
